@@ -4,10 +4,15 @@ package Class::MOP::Attribute;
 use strict;
 use warnings;
 
+use Class::MOP::Method::Accessor;
+
 use Carp         'confess';
 use Scalar::Util 'blessed', 'reftype', 'weaken';
 
-our $VERSION = '0.08';
+our $VERSION   = '0.14';
+our $AUTHORITY = 'cpan:STEVAN';
+
+use base 'Class::MOP::Object';
 
 sub meta { 
     require Class::MOP::Class;
@@ -15,7 +20,7 @@ sub meta {
 }
 
 # NOTE: (meta-circularity)
-# This method will be replaces in the 
+# This method will be replaced in the 
 # boostrap section of Class::MOP, by 
 # a new version which uses the 
 # &Class::MOP::Class::construct_instance
@@ -30,26 +35,36 @@ sub new {
         
     (defined $name && $name)
         || confess "You must provide a name for the attribute";
+        
     $options{init_arg} = $name 
         if not exists $options{init_arg};
             
+    (is_default_a_coderef(\%options))
+        || confess("References are not allowed as default values, you must ". 
+                   "wrap then in a CODE reference (ex: sub { [] } and not [])")
+            if exists $options{default} && ref $options{default};      
+            
     bless {
-        name      => $name,
-        accessor  => $options{accessor},
-        reader    => $options{reader},
-        writer    => $options{writer},
-        predicate => $options{predicate},
-        init_arg  => $options{init_arg},
-        default   => $options{default},
+        '$!name'      => $name,
+        '$!accessor'  => $options{accessor},
+        '$!reader'    => $options{reader},
+        '$!writer'    => $options{writer},
+        '$!predicate' => $options{predicate},
+        '$!clearer'   => $options{clearer},
+        '$!init_arg'  => $options{init_arg},
+        '$!default'   => $options{default},
         # keep a weakened link to the 
         # class we are associated with
-        associated_class => undef,
+        '$!associated_class' => undef,
+        # and a list of the methods 
+        # associated with this attr
+        '@!associated_methods' => [],
     } => $class;
 }
 
 # NOTE:
 # this is a primative (and kludgy) clone operation 
-# for now, it will be repleace in the Class::MOP
+# for now, it will be replaced in the Class::MOP
 # bootstrap with a proper one, however we know 
 # that this one will work fine for now.
 sub clone {
@@ -62,13 +77,13 @@ sub clone {
 
 sub initialize_instance_slot {
     my ($self, $meta_instance, $instance, $params) = @_;
-    my $init_arg = $self->{init_arg};
+    my $init_arg = $self->{'$!init_arg'};
     # try to fetch the init arg from the %params ...
     my $val;        
     $val = $params->{$init_arg} if exists $params->{$init_arg};
     # if nothing was in the %params, we can use the 
     # attribute's default value (if it has one)
-    if (!defined $val && defined $self->{default}) {
+    if (!defined $val && defined $self->{'$!default'}) {
         $val = $self->default($instance);
     }
     $meta_instance->set_slot_value($instance, $self->name, $val);
@@ -78,36 +93,43 @@ sub initialize_instance_slot {
 # the next bunch of methods will get bootstrapped 
 # away in the Class::MOP bootstrapping section
 
-sub name { $_[0]->{name} }
+sub name { $_[0]->{'$!name'} }
 
-sub associated_class { $_[0]->{associated_class} }
+sub associated_class   { $_[0]->{'$!associated_class'}   }
+sub associated_methods { $_[0]->{'@!associated_methods'} }
 
-sub has_accessor  { defined($_[0]->{accessor})  ? 1 : 0 }
-sub has_reader    { defined($_[0]->{reader})    ? 1 : 0 }
-sub has_writer    { defined($_[0]->{writer})    ? 1 : 0 }
-sub has_predicate { defined($_[0]->{predicate}) ? 1 : 0 }
-sub has_init_arg  { defined($_[0]->{init_arg})  ? 1 : 0 }
-sub has_default   { defined($_[0]->{default})   ? 1 : 0 }
+sub has_accessor  { defined($_[0]->{'$!accessor'})  ? 1 : 0 }
+sub has_reader    { defined($_[0]->{'$!reader'})    ? 1 : 0 }
+sub has_writer    { defined($_[0]->{'$!writer'})    ? 1 : 0 }
+sub has_predicate { defined($_[0]->{'$!predicate'}) ? 1 : 0 }
+sub has_clearer   { defined($_[0]->{'$!clearer'})   ? 1 : 0 }
+sub has_init_arg  { defined($_[0]->{'$!init_arg'})  ? 1 : 0 }
+sub has_default   { defined($_[0]->{'$!default'})   ? 1 : 0 }
 
-sub accessor  { $_[0]->{accessor}  } 
-sub reader    { $_[0]->{reader}    }
-sub writer    { $_[0]->{writer}    }
-sub predicate { $_[0]->{predicate} }
-sub init_arg  { $_[0]->{init_arg}  }
+sub accessor  { $_[0]->{'$!accessor'}  } 
+sub reader    { $_[0]->{'$!reader'}    }
+sub writer    { $_[0]->{'$!writer'}    }
+sub predicate { $_[0]->{'$!predicate'} }
+sub clearer   { $_[0]->{'$!clearer'}   }
+sub init_arg  { $_[0]->{'$!init_arg'}  }
 
 # end bootstrapped away method section.
 # (all methods below here are kept intact)
 
+sub is_default_a_coderef { 
+    ('CODE' eq (reftype($_[0]->{'$!default'} || $_[0]->{default}) || ''))    
+}
+
 sub default { 
-    my $self = shift;
-    if (reftype($self->{default}) && reftype($self->{default}) eq 'CODE') {
+    my ($self, $instance) = @_;
+    if (defined $instance && $self->is_default_a_coderef) {
         # if the default is a CODE ref, then 
         # we pass in the instance and default
         # can return a value based on that 
         # instance. Somewhat crude, but works.
-        return $self->{default}->(shift);
+        return $self->{'$!default'}->($instance);
     }           
-    $self->{default};
+    $self->{'$!default'};
 }
 
 # slots
@@ -120,95 +142,110 @@ sub attach_to_class {
     my ($self, $class) = @_;
     (blessed($class) && $class->isa('Class::MOP::Class'))
         || confess "You must pass a Class::MOP::Class instance (or a subclass)";
-    weaken($self->{associated_class} = $class);    
+    weaken($self->{'$!associated_class'} = $class);    
 }
 
 sub detach_from_class {
     my $self = shift;
-    $self->{associated_class} = undef;        
+    $self->{'$!associated_class'} = undef;        
 }
 
-## Method generation helpers
+# method association 
 
-sub generate_accessor_method {
-    my $self = shift; 
-    my $attr_name  = $self->name;
-    return sub {
-        my $meta_instance = Class::MOP::Class->initialize(Scalar::Util::blessed($_[0]))->get_meta_instance;
-        $meta_instance->set_slot_value($_[0], $attr_name, $_[1]) if scalar(@_) == 2;
-        $meta_instance->get_slot_value($_[0], $attr_name);
-    };
+sub associate_method {
+    my ($self, $method) = @_;
+    push @{$self->{'@!associated_methods'}} => $method;
 }
 
-sub generate_reader_method {
-    my $self = shift;
-    my $attr_name  = $self->name;
-    return sub { 
-        confess "Cannot assign a value to a read-only accessor" if @_ > 1;
-        Class::MOP::Class->initialize(Scalar::Util::blessed($_[0]))
-                         ->get_meta_instance
-                         ->get_slot_value($_[0], $attr_name); 
-    };   
+## Slot management
+
+sub set_value {
+    my ($self, $instance, $value) = @_;
+
+    Class::MOP::Class->initialize(blessed($instance))
+                     ->get_meta_instance
+                     ->set_slot_value($instance, $self->name, $value);
 }
 
-sub generate_writer_method {
-    my $self = shift;
-    my $attr_name  = $self->name;
-    return sub { 
-        Class::MOP::Class->initialize(Scalar::Util::blessed($_[0]))
-                         ->get_meta_instance
-                         ->set_slot_value($_[0], $attr_name, $_[1]);
-    };
+sub get_value {
+    my ($self, $instance) = @_;
+
+    Class::MOP::Class->initialize(blessed($instance))
+                     ->get_meta_instance
+                     ->get_slot_value($instance, $self->name);
 }
 
-sub generate_predicate_method {
-    my $self = shift;
-    my $attr_name  = $self->name;
-    return sub { 
-        defined Class::MOP::Class->initialize(Scalar::Util::blessed($_[0]))
-                                 ->get_meta_instance
-                                 ->get_slot_value($_[0], $attr_name) ? 1 : 0;
-    };
+sub has_value {
+    my ($self, $instance) = @_;
+    
+    defined Class::MOP::Class->initialize(blessed($instance))
+                             ->get_meta_instance
+                             ->get_slot_value($instance, $self->name) ? 1 : 0;    
 }
 
+sub clear_value {
+    my ($self, $instance) = @_;
+        
+    Class::MOP::Class->initialize(blessed($instance))
+                     ->get_meta_instance
+                     ->deinitialize_slot($instance, $self->name);    
+}
+
+## load em up ...
+
+sub accessor_metaclass { 'Class::MOP::Method::Accessor' }
+
 sub process_accessors {
-    my ($self, $type, $accessor) = @_;
+    my ($self, $type, $accessor, $generate_as_inline_methods) = @_;
     if (reftype($accessor)) {
         (reftype($accessor) eq 'HASH')
-            || confess "bad accessor/reader/writer/predicate format, must be a HASH ref";
-        my ($name, $method) = each %{$accessor};
-        return ($name, Class::MOP::Attribute::Accessor->wrap($method));        
+            || confess "bad accessor/reader/writer/predicate/clearer format, must be a HASH ref";
+        my ($name, $method) = %{$accessor};
+        $method = $self->accessor_metaclass->wrap($method);
+        $self->associate_method($method);
+        return ($name, $method);        
     }
     else {
-        my $generator = $self->can('generate_' . $type . '_method');
-        ($generator)
-            || confess "There is no method generator for the type='$type'";
-        if (my $method = $self->$generator($self->name)) {
-            return ($accessor => Class::MOP::Attribute::Accessor->wrap($method));            
-        }
-        confess "Could not create the '$type' method for " . $self->name . " because : $@";
+        my $inline_me = ($generate_as_inline_methods && $self->associated_class->instance_metaclass->is_inlinable);         
+        my $method;
+        eval {
+            $method = $self->accessor_metaclass->new(
+                attribute     => $self,
+                is_inline     => $inline_me,
+                accessor_type => $type,
+            );            
+        };
+        confess "Could not create the '$type' method for " . $self->name . " because : $@" if $@;        
+        $self->associate_method($method);
+        return ($accessor, $method);
     }    
 }
 
 sub install_accessors {
-    my $self  = shift;
-    my $class = $self->associated_class;
+    my $self   = shift;
+    my $inline = shift;
+    my $class  = $self->associated_class;
     
     $class->add_method(
-        $self->process_accessors('accessor' => $self->accessor())
+        $self->process_accessors('accessor' => $self->accessor(), $inline)
     ) if $self->has_accessor();
 
     $class->add_method(            
-        $self->process_accessors('reader' => $self->reader())
+        $self->process_accessors('reader' => $self->reader(), $inline)
     ) if $self->has_reader();
 
     $class->add_method(
-        $self->process_accessors('writer' => $self->writer())
+        $self->process_accessors('writer' => $self->writer(), $inline)
     ) if $self->has_writer();
 
     $class->add_method(
-        $self->process_accessors('predicate' => $self->predicate())
+        $self->process_accessors('predicate' => $self->predicate(), $inline)
     ) if $self->has_predicate();
+    
+    $class->add_method(
+        $self->process_accessors('clearer' => $self->clearer(), $inline)
+    ) if $self->has_clearer();
+    
     return;
 }
 
@@ -220,7 +257,7 @@ sub install_accessors {
         }        
         my $method = $class->get_method($accessor);   
         $class->remove_method($accessor) 
-            if (blessed($method) && $method->isa('Class::MOP::Attribute::Accessor'));
+            if (blessed($method) && $method->isa('Class::MOP::Method::Accessor'));
     };
     
     sub remove_accessors {
@@ -229,22 +266,12 @@ sub install_accessors {
         $_remove_accessor->($self->reader(),    $self->associated_class()) if $self->has_reader();
         $_remove_accessor->($self->writer(),    $self->associated_class()) if $self->has_writer();
         $_remove_accessor->($self->predicate(), $self->associated_class()) if $self->has_predicate();
+        $_remove_accessor->($self->clearer(),   $self->associated_class()) if $self->has_clearer();
         return;                        
     }
 
 }
 
-package Class::MOP::Attribute::Accessor;
-
-use strict;
-use warnings;
-
-use Class::MOP::Method;
-
-our $VERSION = '0.01';
-
-our @ISA = ('Class::MOP::Method');
-
 1;
 
 __END__
@@ -358,12 +385,11 @@ an exercise to the reader :).
 
 =back
 
-The I<accessor>, I<reader>, I<writer> and I<predicate> keys can 
-contain either; the name of the method and an appropriate default 
-one will be generated for you, B<or> a HASH ref containing exactly one 
-key (which will be used as the name of the method) and one value, 
-which should contain a CODE reference which will be installed as 
-the method itself.
+The I<accessor>, I<reader>, I<writer>, I<predicate> and I<clearer> keys can
+contain either; the name of the method and an appropriate default one will be
+generated for you, B<or> a HASH ref containing exactly one key (which will be
+used as the name of the method) and one value, which should contain a CODE
+reference which will be installed as the method itself.
 
 =over 4
 
@@ -404,6 +430,11 @@ This is a basic test to see if the value of the attribute is not
 C<undef>. It will return true (C<1>) if the attribute's value is 
 defined, and false (C<0>) otherwise.
 
+=item I<clearer>
+
+This is the a method that will uninitialize the attr, reverting lazy values
+back to their "unfulfilled" state.
+
 =back
 
 =item B<clone (%options)>
@@ -412,6 +443,26 @@ defined, and false (C<0>) otherwise.
 
 =back 
 
+=head2 Value management
+
+=over 4
+
+=item B<set_value ($instance, $value)>
+
+Set the value without going through the accessor. Note that this may be done to
+even attributes with just read only accessors.
+
+=item B<get_value ($instance)>
+
+Return the value without going through the accessor. Note that this may be done
+even to attributes with just write only accessors.
+
+=item B<has_value ($instance)>
+
+=item B<clear_value ($instance)>
+
+=back
+
 =head2 Informational
 
 These are all basic read-only value accessors for the values 
@@ -429,8 +480,12 @@ passed into C<new>. I think they are pretty much self-explanitory.
 
 =item B<predicate>
 
+=item B<clearer>
+
 =item B<init_arg>
 
+=item B<is_default_a_coderef>
+
 =item B<default (?$instance)>
 
 As noted in the documentation for C<new> above, if the I<default> 
@@ -458,6 +513,8 @@ These are all basic predicate methods for the values passed into C<new>.
 
 =item B<has_predicate>
 
+=item B<has_clearer>
+
 =item B<has_init_arg>
 
 =item B<has_default>
@@ -474,18 +531,18 @@ These are all basic predicate methods for the values passed into C<new>.
 
 =item B<detach_from_class>
 
-=item B<slot_name>
-
-=item B<allocate_slots>
-
-=item B<deallocate_slots>
-
 =back
 
 =head2 Attribute Accessor generation
 
 =over 4
 
+=item B<accessor_metaclass>
+
+=item B<associate_method>
+
+=item B<associated_methods>
+
 =item B<install_accessors>
 
 This allows the attribute to generate and install code for it's own 
@@ -503,22 +560,10 @@ different types). It will then either generate the method itself
 (using the C<generate_*_method> methods listed below) or it will 
 use the custom method passed through the constructor. 
 
-=over 4
-
-=item B<generate_accessor_method>
-
-=item B<generate_predicate_method>
-
-=item B<generate_reader_method>
-
-=item B<generate_writer_method>
-
-=back
-
 =item B<remove_accessors>
 
 This allows the attribute to remove the method for it's own 
-I<accessor/reader/writer/predicate>. This is called by 
+I<accessor/reader/writer/predicate/clearer>. This is called by 
 C<Class::MOP::Class::remove_attribute>.
 
 =back
@@ -539,10 +584,12 @@ of the MOP when subclassing it.
 
 =back
 
-=head1 AUTHOR
+=head1 AUTHORS
 
 Stevan Little E<lt>stevan@iinteractive.comE<gt>
 
+Yuval Kogman E<lt>nothingmuch@woobling.comE<gt>
+
 =head1 COPYRIGHT AND LICENSE
 
 Copyright 2006 by Infinity Interactive, Inc.
@@ -552,4 +599,6 @@ L<http://www.iinteractive.com>
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself. 
 
-=cut
\ No newline at end of file
+=cut
+
+
